pub struct ObjectHeader {
pub id: Option<URID>,
pub otype: URID,
}Expand description
Information about an object atom.
Fields
id: Option<URID>The id of the object to distinguish different objects of the same type.
If you don’t need it, you should set it to None.
otype: URIDThe type of the object (same as rdf:type).